@@ -454,8 +454,10 @@ static void get_global_id_and_update(uint64_t *current_id, bool update)
 
 static void wins_hook(const char *operation, struct name_record *namerec, int ttl)
 {
+	const struct loadparm_substitution *lp_sub =
+		loadparm_s3_global_substitution();
 	char *command = NULL;
-	char *cmd = lp_wins_hook(talloc_tos());
+	char *cmd = lp_wins_hook(talloc_tos(), lp_sub);
 	char *p, *namestr;
 	int i;
 	TALLOC_CTX *ctx = talloc_tos();
@@ -1032,7 +1034,7 @@ void wins_process_name_refresh_request( struct subnet_record *subrec,
  We have a locked pointer to the original packet stashed away in the
  userdata pointer. The success here is actually a failure as it means
  the client we queried wants to keep the name, so we must return
- a registration failure to the original requestor.
+ a registration failure to the original requester.
 ************************************************************************/
 
 static void wins_register_query_success(struct subnet_record *subrec,
@@ -1077,7 +1079,7 @@ static void wins_register_query_fail(struct subnet_record *subrec,
 
 	/*
 	 * We want to just add the name, as we now know the original owner
-	 * didn't want it. But we can't just do that as an arbitary
+	 * didn't want it. But we can't just do that as an arbitrary
 	 * amount of time may have taken place between the name query
 	 * request and this timeout/error response. So we check that
 	 * the name still exists and is in the same state - if so
@@ -1546,7 +1548,7 @@ static void wins_multihomed_register_query_success(struct subnet_record *subrec,
 
 	/*
 	 * We want to just add the new IP, as we now know the requesting
-	 * machine claims to own it. But we can't just do that as an arbitary
+	 * machine claims to own it. But we can't just do that as an arbitrary
 	 * amount of time may have taken place between the name query
 	 * request and this response. So we check that
 	 * the name still exists and is in the same state - if so
@@ -1663,7 +1665,7 @@ IP %s\n", nmb_namestr(question), inet_ntoa(from_ip) ));
 
 	if(question->name_type == 0x1d) {
 		DEBUG(3,("wins_process_multihomed_name_registration_request: Ignoring request \
-to register name %s from IP %s.", nmb_namestr(question), inet_ntoa(p->ip) ));
+to register name %s from IP %s.\n", nmb_namestr(question), inet_ntoa(p->ip) ));
 		send_wins_name_registration_response(0, ttl, p);  
 		return;
 	}
@@ -2176,7 +2178,7 @@ IP %s\n", releasing_group_name ? "Group" : "Unique", nmb_namestr(question), inet
 
 	if(!releasing_group_name && (question->name_type == 0x1d)) {
 		DEBUG(3,("wins_process_name_release_request: Ignoring request \
-to release name %s from IP %s.", nmb_namestr(question), inet_ntoa(p->ip) ));
+to release name %s from IP %s.\n", nmb_namestr(question), inet_ntoa(p->ip) ));
 		send_wins_name_release_response(0, p);
 		return;
 	}
