@@ -240,207 +240,6 @@ NTSTATUS rpc_enum_local_groups(TALLOC_CTX *mem_ctx,
 	return NT_STATUS_OK;
 }
 
-/* convert a single name to a sid in a domain */
-NTSTATUS rpc_name_to_sid(TALLOC_CTX *mem_ctx,
-			 struct rpc_pipe_client *lsa_pipe,
-			 struct policy_handle *lsa_policy,
-			 const char *domain_name,
-			 const char *name,
-			 uint32_t flags,
-			 struct dom_sid *sid,
-			 enum lsa_SidType *type)
-{
-	enum lsa_SidType *types = NULL;
-	struct dom_sid *sids = NULL;
-	char *full_name = NULL;
-	const char *names[1];
-	char *mapped_name = NULL;
-	NTSTATUS status;
-
-	if (name == NULL || name[0] == '\0') {
-		full_name = talloc_asprintf(mem_ctx, "%s", domain_name);
-	} else if (domain_name == NULL || domain_name[0] == '\0') {
-		full_name = talloc_asprintf(mem_ctx, "%s", name);
-	} else {
-		full_name = talloc_asprintf(mem_ctx, "%s\\%s", domain_name, name);
-	}
-
-	if (full_name == NULL) {
-		return NT_STATUS_NO_MEMORY;
-	}
-
-	status = normalize_name_unmap(mem_ctx, full_name, &mapped_name);
-	/* Reset the full_name pointer if we mapped anything */
-	if (NT_STATUS_IS_OK(status) ||
-	    NT_STATUS_EQUAL(status, NT_STATUS_FILE_RENAMED)) {
-		full_name = mapped_name;
-	}
-
-	DEBUG(3,("name_to_sid: %s for domain %s\n",
-		 full_name ? full_name : "", domain_name ));
-
-	names[0] = full_name;
-
-	/*
-	 * We don't run into deadlocks here, cause winbind_off() is
-	 * called in the main function.
-	 */
-	status = rpccli_lsa_lookup_names(lsa_pipe,
-					 mem_ctx,
-					 lsa_policy,
-					 1, /* num_names */
-					 names,
-					 NULL, /* domains */
-					 1, /* level */
-					 &sids,
-					 &types);
-	if (!NT_STATUS_IS_OK(status)) {
-		DEBUG(2,("name_to_sid: failed to lookup name: %s\n",
-			nt_errstr(status)));
-		return status;
-	}
-
-	sid_copy(sid, &sids[0]);
-	*type = types[0];
-
-	return NT_STATUS_OK;
-}
-
-/* Convert a domain SID to a user or group name */
-NTSTATUS rpc_sid_to_name(TALLOC_CTX *mem_ctx,
-			 struct rpc_pipe_client *lsa_pipe,
-			 struct policy_handle *lsa_policy,
-			 struct winbindd_domain *domain,
-			 const struct dom_sid *sid,
-			 char **pdomain_name,
-			 char **pname,
-			 enum lsa_SidType *ptype)
-{
-	char *mapped_name = NULL;
-	char **domains = NULL;
-	char **names = NULL;
-	enum lsa_SidType *types = NULL;
-	NTSTATUS map_status;
-	NTSTATUS status;
-
-	status = rpccli_lsa_lookup_sids(lsa_pipe,
-					mem_ctx,
-					lsa_policy,
-					1, /* num_sids */
-					sid,
-					&domains,
-					&names,
-					&types);
-	if (!NT_STATUS_IS_OK(status)) {
-		DEBUG(2,("sid_to_name: failed to lookup sids: %s\n",
-			nt_errstr(status)));
-		return status;
-	}
-
-	*ptype = (enum lsa_SidType) types[0];
-
-	map_status = normalize_name_map(mem_ctx,
-					domain->name,
-					names[0],
-					&mapped_name);
-	if (NT_STATUS_IS_OK(map_status) ||
-	    NT_STATUS_EQUAL(map_status, NT_STATUS_FILE_RENAMED)) {
-		*pname = talloc_strdup(mem_ctx, mapped_name);
-		DEBUG(5,("returning mapped name -- %s\n", *pname));
-	} else {
-		*pname = talloc_strdup(mem_ctx, names[0]);
-	}
-	if ((names[0] != NULL) && (*pname == NULL)) {
-		return NT_STATUS_NO_MEMORY;
-	}
-
-	*pdomain_name = talloc_strdup(mem_ctx, domains[0]);
-	if (*pdomain_name == NULL) {
-		return NT_STATUS_NO_MEMORY;
-	}
-
-	return NT_STATUS_OK;
-}
-
-/* Convert a bunch of rids to user or group names */
-NTSTATUS rpc_rids_to_names(TALLOC_CTX *mem_ctx,
-			   struct rpc_pipe_client *lsa_pipe,
-			   struct policy_handle *lsa_policy,
-			   struct winbindd_domain *domain,
-			   const struct dom_sid *sid,
-			   uint32_t *rids,
-			   size_t num_rids,
-			   char **pdomain_name,
-			   char ***pnames,
-			   enum lsa_SidType **ptypes)
-{
-	enum lsa_SidType *types = NULL;
-	char *domain_name = NULL;
-	char **domains = NULL;
-	char **names = NULL;
-	struct dom_sid *sids;
-	size_t i;
-	NTSTATUS status;
-
-	if (num_rids > 0) {
-		sids = talloc_array(mem_ctx, struct dom_sid, num_rids);
-		if (sids == NULL) {
-			return NT_STATUS_NO_MEMORY;
-		}
-	} else {
-		sids = NULL;
-	}
-
-	for (i = 0; i < num_rids; i++) {
-		if (!sid_compose(&sids[i], sid, rids[i])) {
-			return NT_STATUS_INTERNAL_ERROR;
-		}
-	}
-
-	status = rpccli_lsa_lookup_sids(lsa_pipe,
-					mem_ctx,
-					lsa_policy,
-					num_rids,
-					sids,
-					&domains,
-					&names,
-					&types);
-	if (!NT_STATUS_IS_OK(status) &&
-	    !NT_STATUS_EQUAL(status, STATUS_SOME_UNMAPPED)) {
-		DEBUG(2,("rids_to_names: failed to lookup sids: %s\n",
-			nt_errstr(status)));
-		return status;
-	}
-
-	for (i = 0; i < num_rids; i++) {
-		char *mapped_name = NULL;
-		NTSTATUS map_status;
-
-		if (types[i] != SID_NAME_UNKNOWN) {
-			map_status = normalize_name_map(mem_ctx,
-							domain->name,
-							names[i],
-							&mapped_name);
-			if (NT_STATUS_IS_OK(map_status) ||
-			    NT_STATUS_EQUAL(map_status, NT_STATUS_FILE_RENAMED)) {
-				TALLOC_FREE(names[i]);
-				names[i] = talloc_strdup(names, mapped_name);
-				if (names[i] == NULL) {
-					return NT_STATUS_NO_MEMORY;
-				}
-			}
-
-			domain_name = domains[i];
-		}
-	}
-
-	*pdomain_name = domain_name;
-	*ptypes = types;
-	*pnames = names;
-
-	return NT_STATUS_OK;
-}
-
 /* Lookup groups a user is a member of. */
 NTSTATUS rpc_lookup_usergroups(TALLOC_CTX *mem_ctx,
 			       struct rpc_pipe_client *samr_pipe,
@@ -692,48 +491,6 @@ NTSTATUS rpc_lookup_groupmem(TALLOC_CTX *mem_ctx,
 
 		break;
 	}
-	case SID_NAME_WKN_GRP:
-	case SID_NAME_ALIAS:
-	{
-		struct lsa_SidArray sid_array;
-		struct lsa_SidPtr sid_ptr;
-		struct samr_Ids rids_query;
-
-		sid_ptr.sid = dom_sid_dup(mem_ctx, group_sid);
-		if (sid_ptr.sid == NULL) {
-			return NT_STATUS_NO_MEMORY;
-		}
-
-		sid_array.num_sids = 1;
-		sid_array.sids = &sid_ptr;
-
-		status = dcerpc_samr_GetAliasMembership(b,
-							mem_ctx,
-							samr_policy,
-							&sid_array,
-							&rids_query,
-							&result);
-		if (!NT_STATUS_IS_OK(status)) {
-			return status;
-		}
-		if (!NT_STATUS_IS_OK(result)) {
-			return result;
-		}
-
-		if (rids_query.count == 0) {
-			pnum_names = 0;
-			pnames = NULL;
-			pname_types = NULL;
-			psid_mem = NULL;
-
-			return NT_STATUS_OK;
-		}
-
-		num_names = rids_query.count;
-		rid_mem = rids_query.ids;
-
-		break;
-	}
 	default:
 		return NT_STATUS_UNSUCCESSFUL;
 	}
@@ -807,63 +564,79 @@ NTSTATUS rpc_lookup_groupmem(TALLOC_CTX *mem_ctx,
 	return NT_STATUS_OK;
 }
 
-/* Find the sequence number for a domain */
-NTSTATUS rpc_sequence_number(TALLOC_CTX *mem_ctx,
+/* Lookup alias membership using a rid taken from alias_sid. */
+NTSTATUS rpc_lookup_aliasmem(TALLOC_CTX *mem_ctx,
 			     struct rpc_pipe_client *samr_pipe,
 			     struct policy_handle *samr_policy,
-			     const char *domain_name,
-			     uint32_t *pseq)
+			     const struct dom_sid *domain_sid,
+			     const struct dom_sid *alias_sid,
+			     enum lsa_SidType type,
+			     uint32_t *pnum_sids,
+			     struct dom_sid **psids)
 {
-	union samr_DomainInfo *info = NULL;
-	bool got_seq_num = false;
+	uint32_t alias_rid;
+	struct dom_sid *sid_mem = NULL;
+	struct lsa_SidArray sid_array;
+	uint32_t i;
 	NTSTATUS status, result;
 	struct dcerpc_binding_handle *b = samr_pipe->binding_handle;
 
-	/* query domain info */
-	status = dcerpc_samr_QueryDomainInfo(b,
-					     mem_ctx,
-					     samr_policy,
-					     8,
-					     &info,
-					     &result);
-	if (NT_STATUS_IS_OK(status) && NT_STATUS_IS_OK(result)) {
-		*pseq = info->info8.sequence_num;
-		got_seq_num = true;
-		goto seq_num;
+	if (!sid_peek_check_rid(domain_sid, alias_sid, &alias_rid)) {
+		return NT_STATUS_UNSUCCESSFUL;
 	}
 
-	/* retry with info-level 2 in case the dc does not support info-level 8
-	 * (like all older samba2 and samba3 dc's) - Guenther */
-	status = dcerpc_samr_QueryDomainInfo(b,
-					     mem_ctx,
-					     samr_policy,
-					     2,
-					     &info,
-					     &result);
-	if (NT_STATUS_IS_OK(status) && NT_STATUS_IS_OK(result)) {
-		*pseq = info->general.sequence_num;
-		got_seq_num = true;
-		goto seq_num;
+	switch (type) {
+	case SID_NAME_ALIAS: {
+		struct policy_handle alias_policy;
+
+		status = dcerpc_samr_OpenAlias(b,
+					       mem_ctx,
+					       samr_policy,
+					       SEC_FLAG_MAXIMUM_ALLOWED,
+					       alias_rid,
+					       &alias_policy,
+					       &result);
+		if (any_nt_status_not_ok(status, result, &status)) {
+			return status;
+		}
+
+		status = dcerpc_samr_GetMembersInAlias(b,
+						       mem_ctx,
+						       &alias_policy,
+						       &sid_array,
+						       &result);
+		{
+			NTSTATUS _result;
+			dcerpc_samr_Close(b, mem_ctx, &alias_policy, &_result);
+		}
+		if (any_nt_status_not_ok(status, result, &status)) {
+			return status;
+		}
+
+		sid_mem = talloc_zero_array(mem_ctx,
+					    struct dom_sid,
+					    sid_array.num_sids);
+		if (sid_mem == NULL) {
+			return NT_STATUS_NO_MEMORY;
+		}
+
+		/*
+		 * We cannot just simply assign '*psids = sid_array.sids;'
+		 * we need to copy every sid since these are incompatible types:
+		 * 'struct dom_sid *' vs 'struct lsa_SidPtr *'
+		 */
+		for (i = 0; i < sid_array.num_sids; i++) {
+			sid_copy(&sid_mem[i], sid_array.sids[i].sid);
+		}
+
+		*pnum_sids = sid_array.num_sids;
+		*psids = sid_mem;
+
+		return NT_STATUS_OK;
 	}
-
-	if (!NT_STATUS_IS_OK(status)) {
-		goto seq_num;
+	default:
+		return NT_STATUS_UNSUCCESSFUL;
 	}
-
-	status = result;
-
-seq_num:
-	if (got_seq_num) {
-		DEBUG(10,("domain_sequence_number: for domain %s is %u\n",
-			  domain_name, (unsigned) *pseq));
-	} else {
-		DEBUG(10,("domain_sequence_number: failed to get sequence "
-			  "number (%u) for domain %s\n",
-			  (unsigned) *pseq, domain_name ));
-		status = NT_STATUS_OK;
-	}
-
-	return status;
 }
 
 /* Get a list of trusted domains */
@@ -939,26 +712,24 @@ NTSTATUS rpc_trusted_domains(TALLOC_CTX *mem_ctx,
 				return NT_STATUS_NO_MEMORY;
 			}
 
+			if (dom_list_ex.domains[i].sid == NULL) {
+				DBG_ERR("Trusted domain %s has no SID, "
+					"skipping!\n",
+					trust->dns_name);
+				continue;
+			}
+
 			if (has_ex) {
 				trust->netbios_name = talloc_move(array,
 								  &dom_list_ex.domains[i].netbios_name.string);
 				trust->dns_name = talloc_move(array,
 							      &dom_list_ex.domains[i].domain_name.string);
-				if (dom_list_ex.domains[i].sid == NULL) {
-					DEBUG(0, ("Trusted Domain %s has no SID, aborting!\n", trust->dns_name));
-					return NT_STATUS_INVALID_NETWORK_RESPONSE;
-				}
 				sid_copy(sid, dom_list_ex.domains[i].sid);
 			} else {
 				trust->netbios_name = talloc_move(array,
 								  &dom_list.domains[i].name.string);
 				trust->dns_name = NULL;
 
-				if (dom_list.domains[i].sid == NULL) {
-					DEBUG(0, ("Trusted Domain %s has no SID, aborting!\n", trust->netbios_name));
-					return NT_STATUS_INVALID_NETWORK_RESPONSE;
-				}
-
 				sid_copy(sid, dom_list.domains[i].sid);
 			}
 
