@@ -28,7 +28,7 @@
 #include "librpc/gen_ndr/ndr_nbt.h"
 #include "system/time.h"
 #include "libcli/composite/composite.h"
-#include "smbd/service_task.h"
+#include "samba/service_task.h"
 #include "system/network.h"
 #include "lib/socket/socket.h"
 #include "lib/socket/netif.h"
@@ -36,6 +36,7 @@
 #include "param/param.h"
 #include "libcli/resolve/resolve.h"
 #include "lib/util/util_net.h"
+#include "lib/util/tsort.h"
 
 /*
   work out the ttl we will use given a client requested ttl
@@ -422,7 +423,7 @@ static void wins_register_wack(struct nbt_name_socket *nbtsock,
 	s->io.in.addresses	= winsdb_addr_string_list(s, rec->addresses);
 	if (s->io.in.addresses == NULL) goto failed;
 
-	DLIST_ADD_END(iface->wack_queue, s, struct nbtd_wins_wack_state *);
+	DLIST_ADD_END(iface->wack_queue, s);
 
 	talloc_set_destructor(s, nbtd_wins_wack_state_destructor);
 
@@ -653,7 +654,7 @@ static int nbtd_wins_randomize1Clist_sort(void *p1,/* (const char **) */
 	match_bits1 = ipv4_match_bits(interpret_addr2(a1), interpret_addr2(src->addr));
 	match_bits2 = ipv4_match_bits(interpret_addr2(a2), interpret_addr2(src->addr));
 
-	return match_bits2 - match_bits1;
+	return NUMERIC_CMP(match_bits2, match_bits1);
 }
 
 static void nbtd_wins_randomize1Clist(struct loadparm_context *lp_ctx,
@@ -679,7 +680,7 @@ static void nbtd_wins_randomize1Clist(struct loadparm_context *lp_ctx,
 
 	/* 
 	 * choose a random address to be the first in the response to the client,
-	 * preferr the addresses inside the nbtd:wins_randomize1Clist_mask netmask
+	 * prefer the addresses inside the nbtd:wins_randomize1Clist_mask netmask
 	 */
 	r = random();
 	idx = sidx = r % num_addrs;
